﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Diagnostics;

namespace Network
{
    public class UDPSocket : UDP
    {
        private UdpState mState = null;
        private UdpClient mUDP;
        private IPEndPoint mBroadcast;

        public UDPSocket(NetworkInterface I, int BindPort, int Port)
            : base(I, Port)
        {
            var mBroadcastAddress = I.Address.GetAddressBytes();
            var mMask = (I.Mask != null) ? I.Mask.GetAddressBytes() : new byte[] { 0, 0, 0, 0 };

            for (int i = 0; i < 4; i++)
                mBroadcastAddress[i] |= Convert.ToByte(~mMask[i] & 0xFF);

            mBroadcast = new IPEndPoint(new IPAddress(mBroadcastAddress), Port);
            BindClient(I.Address, BindPort);
        }

        public UDPSocket(NetworkInterface I, int Port)
            : base (I, Port)
        {
            var mBroadcastAddress = I.Address.GetAddressBytes();
            var mMask = (I.Mask != null) ? I.Mask.GetAddressBytes() : new byte[] { 0, 0, 0, 0 };

            for (int i = 0; i < 4; i++)
                mBroadcastAddress[i] |= Convert.ToByte(~mMask[i] & 0xFF);

            mBroadcast = new IPEndPoint(new IPAddress(mBroadcastAddress), Port);
            BindClient(I.Address, Port);
        }

        private class UdpState
        {
            public IPEndPoint e;
            public UdpClient u;
            public bool StopRequest = false;

            public UdpState(IPEndPoint E, UdpClient C)
            {
                e = E;
                u = C;
            }
        }

        /// <summary>
        /// Адресный пакет
        /// </summary>
        /// <param name="EP"></param>
        /// <param name="Packet"></param>
        public override void Send(IPEndPoint EP, byte[] Packet)
        {
            mUDP.Send(Packet, Packet.Length, EP);
        }

        /// <summary>
        /// Адресный пакет
        /// </summary>
        /// <param name="IP"></param>
        /// <param name="Packet"></param>
        public override void Send(IPAddress IP, byte[] Packet)
        {
            mUDP.Send(Packet, Packet.Length, new IPEndPoint(IP, mPort));
        }

        /// <summary>
        /// Адресный пакет
        /// </summary>
        /// <param name="IP"></param>
        /// <param name="Packet"></param>
        public override void Send(IPAddress IP, byte[] Packet, int Port)
        {
            mUDP.Send(Packet, Packet.Length, new IPEndPoint(IP, Port));
        }


        /// <summary>
        /// Широковещательный пакет
        /// </summary>
        /// <param name="EP"></param>
        /// <param name="Packet"></param>
        public override void Send(byte[] Packet)
        {
            mUDP.Send(Packet, Packet.Length, mBroadcast);
        }

        /// <summary>
        /// Переподключить
        /// </summary>
        /// <param name="I"></param>
        protected override void RebindInternal(NetworkInterface I, int Port)
        {
            if (mState != null)
            {
                mState.StopRequest = true;
            }
            BindClient(I.Address, Port);
        }

        /// <summary>
        /// Подключить приёмник
        /// </summary>
        /// <param name="A"></param>
        /// <param name="Port"></param>
        private void BindClient(IPAddress A, int Port)
        {
            Debug.WriteLine("Bind to " + A.ToString() + ":" + Port.ToString());
            IPEndPoint localEp = new IPEndPoint(A, Port);
            UdpClient U = new UdpClient();
            U.ExclusiveAddressUse = false;
            U.EnableBroadcast = true;
            U.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);
            U.Client.Bind(localEp);

            mState = new UdpState(localEp, U);
            U.BeginReceive(new AsyncCallback(DataReceived), mState);
            mUDP = U;
        }

        /// <summary>
        /// Принят пакет
        /// </summary>
        /// <param name="R"></param>
        private void DataReceived(IAsyncResult R)
        {
            UdpState S = R.AsyncState as UdpState;
            Byte[] data = null;

            try
            {
                if (S.u != null)
                {
                    data = S.u.EndReceive(R, ref S.e);

                    if (S.StopRequest)
                    {
                        S.u.Close();
                    }
                    else
                    {
                        S.u.BeginReceive(new AsyncCallback(DataReceived), S);
                        if (data != null) fireOnPacket(S.e.Address, data);
                    }
                }
            }
            catch(ObjectDisposedException E)
            {
                Debug.WriteLine("UDPSocket: " + E.Message);
            }
            catch (Exception E)
            {
                Debug.WriteLine("UDPSocket: " + E.Message);
            }
        }
    }
}
